@@ -1,8 +1,9 @@
+from __future__ import print_function
 import unittest
 import timeit
 import sys
 from bitstruct import *
-
+import bitstruct
 
 class BitStructTest(unittest.TestCase):
 
@@ -21,6 +22,14 @@ class BitStructTest(unittest.TestCase):
         ref = b'\x00\x80\x00\x00\x00\x00\x08\x00\x00\x00'
         self.assertEqual(packed, ref)
 
+        packed = pack('u8000', int(8000 * '1', 2))
+        ref = 1000 * b'\xff'
+        self.assertEqual(packed, ref)
+
+        packed = pack('s4000', int(8000 * '0', 2))
+        ref = 500 * b'\x00'
+        self.assertEqual(packed, ref)
+
         packed = pack('p1u1s6u7u9', 0, -2, 65, 22)
         self.assertEqual(packed, b'\x3e\x82\x16')
 
@@ -45,6 +54,9 @@ class BitStructTest(unittest.TestCase):
         packed = pack('b1t24', False, "Hi!")
         self.assertEqual(packed, b'$4\x90\x80')
 
+        packed = pack('t8000', 1000 * "7")
+        self.assertEqual(packed, 1000 * b'\x37')
+
         # Too many values to pack.
         try:
             pack('b1t24', False)
@@ -104,12 +116,20 @@ class BitStructTest(unittest.TestCase):
         self.assertEqual(unpacked, (0, 0, -2, 65, 22))
 
         unpacked = unpack('u1', bytearray(b'\x80'))
-        self.assertEqual(unpacked, (1,))
+        self.assertEqual(unpacked, (1, ))
 
         packed = b'\x00\x80\x00\x00\x00\x00\x08\x00\x00\x00'
         unpacked = unpack('u77', packed)
         self.assertEqual(unpacked, (0x100000000001000000,))
 
+        packed = 1000 * b'\xff'
+        unpacked = unpack('u8000', packed)
+        self.assertEqual(unpacked, (int(8000 * '1', 2), ))
+
+        packed = 500 * b'\x00'
+        unpacked = unpack('s4000', packed)
+        self.assertEqual(unpacked, (0, ))
+
         packed = b'\x3e\x82\x16'
         unpacked = unpack('p1u1s6u7u9', packed)
         self.assertEqual(unpacked, (0, -2, 65, 22))
@@ -124,7 +144,7 @@ class BitStructTest(unittest.TestCase):
 
         packed = [0x80]
         unpacked = unpack('b1', packed)
-        self.assertEqual(unpacked, (True,))
+        self.assertEqual(unpacked, (True, ))
 
         packed = b'\x80'
         unpacked = unpack('b1p6b1', packed)
@@ -142,6 +162,10 @@ class BitStructTest(unittest.TestCase):
         unpacked = unpack('b1t24', packed)
         self.assertEqual(unpacked, (False, u"Hi!"))
 
+        packed = 1000 * b"7"
+        unpacked = unpack('t8000', packed)
+        self.assertEqual(packed, 1000 * b'\x37')
+
         # Bad float size.
         try:
             unpack('f33', b'\x00\x00\x00\x00\x00')
@@ -149,7 +173,7 @@ class BitStructTest(unittest.TestCase):
         except ValueError as e:
             self.assertEqual(
                 str(e),
-                'expected float size of 32 of 64 bits (got 33)')
+                'expected float size of 16, 32, or 64 bits (got 33)')
 
         # Too many bits to unpack.
         try:
@@ -191,6 +215,11 @@ class BitStructTest(unittest.TestCase):
         unpacked = unpack('f64', packed)
         self.assertEqual(unpacked, (1.0, ))
 
+        if sys.version_info >= (3, 6):
+            packed = pack('f16', 1.0)
+            unpacked = unpack('f16', packed)
+            self.assertEqual(unpacked, (1.0, ))
+
     def test_calcsize(self):
         """Calculate size.
 
@@ -318,11 +347,13 @@ class BitStructTest(unittest.TestCase):
         unpacked = unpack('u19u5u1u7<', packed)
         self.assertEqual(unpacked, (0x12345, 5, 1, 2))
 
-    def test_performance(self):
-        """Test pack/unpack performance.
+    def test_performance_mixed_types(self):
+        """Test pack/unpack performance with mixed types.
 
         """
 
+        print()
+
         time = timeit.timeit("pack('s6u7r40b1t152', "
                              "-2, 22, b'\x01\x01\x03\x04\x05', "
                              "True, u'foo fie bar gom gum')",
@@ -330,6 +361,13 @@ class BitStructTest(unittest.TestCase):
                              number=50000)
         print("pack time: {} s ({} s/pack)".format(time, time / 50000))
 
+        time = timeit.timeit(
+            "fmt.pack(-2, 22, b'\x01\x01\x03\x04\x05', "
+            "True, u'foo fie bar gom gum')",
+            setup="import bitstruct ; fmt = bitstruct.compile('s6u7r40b1t152')",
+            number=50000)
+        print("pack time compiled: {} s ({} s/pack)".format(time, time / 50000))
+
         time = timeit.timeit("unpack('s6u7r40b1t152', "
                              "b'\\xf8\\xb0\\x08\\x08\\x18 "
                              "-\\x99\\xbd\\xbc\\x81\\x99"
@@ -340,6 +378,165 @@ class BitStructTest(unittest.TestCase):
                              number=50000)
         print("unpack time: {} s ({} s/unpack)".format(time, time / 50000))
 
+        time = timeit.timeit(
+            "fmt.unpack(b'\\xf8\\xb0\\x08\\x08\\x18 "
+            "-\\x99\\xbd\\xbc\\x81\\x99"
+            "\\xa5\\x94\\x81\\x89\\x85"
+            "\\xc8\\x81\\x9d\\xbd\\xb4"
+            "\\x81\\x9d\\xd5\\xb4')",
+            setup="import bitstruct ; fmt = bitstruct.compile('s6u7r40b1t152')",
+            number=50000)
+        print("unpack time compiled: {} s ({} s/unpack)".format(time, time / 50000))
+
+    def test_performance_integers(self):
+        """Test pack/unpack performance with integers.
+
+        """
+
+        print()
+
+        time = timeit.timeit("pack('s13u7u35u1s9', "
+                             "-2, 22, 44567233, 0, 33)",
+                             setup="from bitstruct import pack",
+                             number=50000)
+        print("pack time: {} s ({} s/pack)".format(time, time / 50000))
+
+        time = timeit.timeit(
+            "fmt.pack(-2, 22, 44567233, 0, 33)",
+            setup="import bitstruct ; fmt = bitstruct.compile('s13u7u35u1s9')",
+            number=50000)
+        print("pack time compiled: {} s ({} s/pack)".format(time, time / 50000))
+
+        time = timeit.timeit("unpack('s13u7u35u1s9', "
+                             "b'\\xff\\xf1`\\x05P\\x15\\x82\\x10\\x80')",
+                             setup="from bitstruct import unpack",
+                             number=50000)
+        print("unpack time: {} s ({} s/unpack)".format(time, time / 50000))
+
+        time = timeit.timeit(
+            "fmt.unpack(b'\\xff\\xf1`\\x05P\\x15\\x82\\x10\\x80')",
+            setup="import bitstruct ; fmt = bitstruct.compile('s13u7u35u1s9')",
+            number=50000)
+        print("unpack time compiled: {} s ({} s/unpack)".format(time, time / 50000))
+
+    def test_compile(self):
+        cf = bitstruct.compile('u1u1s6u7u9')
+
+        packed = cf.pack(0, 0, -2, 65, 22)
+        self.assertEqual(packed, b'\x3e\x82\x16')
+
+        unpacked = cf.unpack(b'\x3e\x82\x16')
+        self.assertEqual(unpacked, (0, 0, -2, 65, 22))
+
+    def test_signed_integer(self):
+        """Pack and unpack signed integer values.
+
+        """
+
+        datas = [
+            ('s2', 0x01, b'\x40'),
+            ('s3', 0x03, b'\x60'),
+            ('s4', 0x07, b'\x70'),
+            ('s5', 0x0f, b'\x78'),
+            ('s6', 0x1f, b'\x7c'),
+            ('s7', 0x3f, b'\x7e'),
+            ('s8', 0x7f, b'\x7f'),
+            ('s9', 0xff, b'\x7f\x80'),
+            ('s1',   -1, b'\x80'),
+            ('s2',   -1, b'\xc0')
+        ]
+
+        for fmt, value, packed in datas:
+            self.assertEqual(pack(fmt, value), packed)
+            self.assertEqual(unpack(fmt, packed), (value, ))
+
+    def test_unsigned_integer(self):
+        """Pack and unpack unsigned integer values.
+
+        """
+
+        datas = [
+            ('u1', 0x001, b'\x80'),
+            ('u2', 0x003, b'\xc0'),
+            ('u3', 0x007, b'\xe0'),
+            ('u4', 0x00f, b'\xf0'),
+            ('u5', 0x01f, b'\xf8'),
+            ('u6', 0x03f, b'\xfc'),
+            ('u7', 0x07f, b'\xfe'),
+            ('u8', 0x0ff, b'\xff'),
+            ('u9', 0x1ff, b'\xff\x80')
+        ]
+
+        for fmt, value, packed in datas:
+            self.assertEqual(pack(fmt, value), packed)
+            self.assertEqual(unpack(fmt, packed), (value, ))
+
+    def test_bad_float_size(self):
+        """Test of bad float size.
+
+        """
+
+        with self.assertRaises(ValueError) as cm:
+            pack('f31', 1.0)
+
+        self.assertEqual(str(cm.exception),
+                         'expected float size of 16, 32, or 64 bits (got 31)')
+
+        with self.assertRaises(ValueError) as cm:
+            unpack('f33', 8 * b'\x00')
+
+        self.assertEqual(str(cm.exception),
+                         'expected float size of 16, 32, or 64 bits (got 33)')
+
+    def test_bad_format(self):
+        """Test of bad format.
+
+        """
+
+        formats = [
+            ('g1', "bad char 'g' in format"),
+            ('s1u1f32b1t8r8G13', "bad char 'G' in format"),
+            ('s1u1f32b1t8r8G13S3', "bad char 'G' in format"),
+            ('s', "bad format 's'"),
+            ('1', "bad format '1'"),
+            ('ss1', "bad format 'ss1'"),
+            ('1s', "bad format '1s'"),
+            ('foo', "bad format 'foo'"),
+            ('s>1>', "bad format 's>1>'")
+        ]
+
+        for fmt, expected_error in formats:
+            with self.assertRaises(ValueError) as cm:
+                bitstruct.compile(fmt)
+
+            self.assertEqual(str(cm.exception), expected_error)
+
+    def test_empty_format(self):
+        """Test of empty format type.
+
+        """
+
+        cf = bitstruct.compile('')
+
+        self.assertEqual(cf.pack(), b'')
+        self.assertEqual(cf.pack(1), b'')
+
+        self.assertEqual(cf.unpack(b''), ())
+        self.assertEqual(cf.unpack(b'\x00'), ())
+
+    def test_byte_order_format(self):
+        """Test of a format with only byte order informationx.
+
+        """
+
+        cf = bitstruct.compile('>')
+
+        self.assertEqual(cf.pack(), b'')
+        self.assertEqual(cf.pack(1), b'')
+
+        self.assertEqual(cf.unpack(b''), ())
+        self.assertEqual(cf.unpack(b'\x00'), ())
+
 
 if __name__ == '__main__':
     unittest.main()
